<?php
/*+********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is: vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 *********************************************************************************/

class Vtiger_UiData_Action extends Vtiger_BasicAjax_Action {
    
	function __construct() {
		parent::__construct();
        $this->exposeMethod('jsStrings');
	}

    public function process(\Vtiger_Request $request) {
        $mode = $request->get('mode');
		if(!empty($mode)) {
			$this->invokeExposedMethod($mode, $request);
			return;
		}
    }
    
    /**
     * Function to set header details
     */
    protected function writeOutput($content, $content_type, $seconds_to_cache = null){
        if ($seconds_to_cache === null) {
            $cache_max_age = PerformancePrefs::getInteger('UI_DATA_MAX_AGE', 1);
            $seconds_to_cache = 3600 * 24 * $cache_max_age; /* X day */
        }

        /* Cache friendly headers */
        $ts = gmdate("D, d M Y H:i:s", time() + $seconds_to_cache) . " GMT";
        header('Expires: '.$ts);
        header('Pragma: cache');
        header('Cache-control: public, max-age = '.$seconds_to_cache . ', immutable'); /* immutable for FF/Edge */
        
        header("Content-Type: $content_type; charset=utf-8");
        echo $content;
    }
    
    /**
     * Function to load JS strings on-demand
     */
    function jsStrings(Vtiger_Request $request) {
        $moduleName = $request->getModule(false);
		if ($moduleName === 'Settings:Users') {
			$moduleName = 'Users';
		}
        $jsLanguageStrings = JSON_encode(Vtiger_Language_Handler::export($moduleName, 'jsLanguageStrings'));
        $this->writeOutput($jsLanguageStrings, "text/plain"); // To make it friendly to Gzip (decode on the client-browser)
    }
}