<?php
/*+********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is: vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 *********************************************************************************/

class Vtiger_Versions_Runtime extends Vtiger_Base_Model {
    private static $instance = false;
    const TABLE = 'vtiger_system_runtime_versions';

    static function getInstance() {
        if(!self::$instance) {
            $instance = new self();

            $versions = Vtiger_Cache::get('RuntimeVersions', 'Versions');
            $versions = ($versions) ? json_decode($versions, true) : '';
            if(empty($versions)) {
                $db = PearDatabase::getInstance();
                $query = "SELECT * FROM ". self::TABLE;
                $result = $db->pquery($query, array());
                $count = $db->num_rows($result);

                $versions = array();
                for($row = 0; $row < $count; $row++) {
                    $rowdata = $db->query_result_rowdata($result, $row);
                    $versions[$rowdata['path']] = $rowdata['version'];
                }

                Vtiger_Cache::set('RuntimeVersions', 'Versions', json_encode($versions));
            }

            $instance->setData($versions);
            self::$instance = $instance;
        }

        return self::$instance;
    }

    function save($path) {
        global $vtiger_current_version;

        $db = PearDatabase::getInstance();
        $query = "INSERT INTO ". self::TABLE . "(path, version) "
                  ."VALUES(?,?) "
                  ."ON DUPLICATE KEY UPDATE "
                  ."version=VALUES(version)";
        
        $version = $vtiger_current_version.'.'.date('dmYhis');
        $db->pquery($query, array($path, $version));

        $versions = $this->getData();
        $versions[$path] = $version;
        Vtiger_Cache::set('RuntimeVersions', 'Versions', json_encode($versions));
    }

    function update($path) {
        if(!$path) return;

        $path = str_replace('/', '.', $path);
        $this->save($path);
    }

    function create($path) {
        if(!$path) return;
    
        $path = str_replace('/', '.', $path);
        $this->save($path);
    }

    function delete($path) {
        if(!$path) return;

        $path = str_replace('/', '.', $path);
        $db = PearDatabase::getInstance();
        $query = "DELETE FROM ".self::TABLE." WHERE path=?";
        $db->pquery($query, array($path));

        $versions = $this->getData();
        unset($versions[$path]);
        Vtiger_Cache::set('RuntimeVersions', 'Versions', json_encode($versions));
    }

}